// Lawson Labs, Inc.
// 74 4th Ave. NW
// Kalispell, MT  59901
// 800 321-5355 or 406 257-5355                           January 23, 1996

#define STRICT
#include <windows.h>
#include <stdio.h>
#include <alloc.h>
#include <stdlib.h>
#include <conio.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <dos.h>
#ifdef __cplusplus
	 #define __CPPARGS ...
#else
	 #define __CPPARGS
#endif


//typedef for M134 struct
typedef struct{
unsigned int    adc134;           //ADC for Model 134
double          hertzRate;        //scan rate (hz)
unsigned int    average;          //number of samples to average per point
unsigned int    ichDelay;         //interchannel delay
unsigned int    numData;          //number of datapoints per scan
unsigned long   totScans;         //number of scans to perform
unsigned int    digInp;           //flag for digital input 1 = on  0 = off
BYTE            controlCode;      //control code for next conversion in GetConv
unsigned int    cCode[65];        //array of control codes
unsigned int    interruptFlag;    //1 = interrupts enabled, 0 = interrupts disabled
unsigned long   numCompleted;     //number of scans completed if scan was cut short
unsigned long   overHead;         //time in ticks for overhead
}       T_M134;

//local prototypes
void         SetWindowToDesktop(void);
short        CheckADC(unsigned int adc134);
void         PrintCommandMenu(T_M134 far *M134);
void         DoCommand(T_M134 far *M134, char ch, double *max, double *min);
void         StandardDev(double *stDv,
								 double *rmsBits,
								 double *sdTot,
								 double *runningAvg,
								 double *sd,
								 double *voltage,
						       int    *sdCount);
unsigned int GetAvg(unsigned int average);
void         SaveData(T_M134 far *M134, unsigned int huge *dataBuffer);
void         DispCodes(T_M134 far *M134);
short        SetupScan(T_M134 far *M134, double *maxHertzRate);
short        ReadConfig(T_M134 far *M134);
short        WriteConfig(T_M134 far *M134);

//prototypes for external DLL functions
short  (FAR PASCAL *ADInit)(int adc, BYTE cCode); //test for existance of 134 at ADC address
double (FAR PASCAL *GetConv)(T_M134 far *M134);   //get one conversion and digital input status
short  (FAR PASCAL *Call134)(T_M134 far *M134, unsigned int huge *clientDataBuffer);  //get conversions based on the T_M134 structure
short  (FAR PASCAL *OverHead)(T_M134 far *M134); //find overhead for getting conversions

//constants
const double tickTime = 8.380953e-7; //time in seconds for each tick (timer resolution)
const convTime = 0x56;               //time for conversion in tick counts

//Global Variables
HINSTANCE       hLibrary;

int main(void)
{
T_M134            M134;
unsigned int huge *StoreConv;     	//pointer for storage of conversions
HANDLE            hglbStoreConv;
double            scanTime, maxHertzRate, voltage, max=-6, min=6;
unsigned long     arraySize;        //size of storage area in bytes
unsigned long     numScans;
long              i, j;
int					n, p_row, p_col, incFlag, endDisplay, cStat;
char              tmpbuf[80], ch;
double            xxT;
clock_t           xx;
//variables for standard deviation calculation
double            stDv=0, rmsBits=0, sdTot = 0, runningAvg=0, sd[41];;
int               sdCount=1;

	hglbStoreConv = NULL;
	SetWindowToDesktop();

	hLibrary = LoadLibrary ("134DLL.DLL");

	(FARPROC) ADInit =GetProcAddress(hLibrary, "ADInit");
	(FARPROC) GetConv =GetProcAddress(hLibrary, "GetConv");
	(FARPROC) Call134 =GetProcAddress(hLibrary, "Call134");
	(FARPROC) OverHead =GetProcAddress(hLibrary, "OverHead");

	if (!ReadConfig(&M134))
	{
	  gotoxy(24,10);
	  printf("unable to read 134EWIN.CFG file");
	  gotoxy(26,11);
	  printf("press any key to continue");
	  getch();
	}

TOP_MENU:
  if (hglbStoreConv != NULL)
  {
	 GlobalUnlock(hglbStoreConv);
	 GlobalFree(hglbStoreConv);
  }

  ch = 0;

  while(ch != 27)
  {
	 clrscr();
	 gotoxy(27,4);
	 printf ("MODEL 134");
	 gotoxy(27,6);
	 printf ("Main Menu");
	 gotoxy(20,8);
	 printf ("1) ADC              %u",M134.adc134);
	 gotoxy(20,10);
	 printf ("2) Polled Mode Operation");
	 gotoxy(20,12);
	 printf ("3) Scanning Mode Operation");
	 gotoxy(18,14);
	 printf ("Esc) Exit program");

	 ch = getch();

	 if (ch == 27)
	 {
		goto EXIT_MAIN;
	 }

	 if (ch == '1')
	 {
GET_ADC:
		clrscr();
		printf("Enter ADC (check your manual) ");
		fgets(tmpbuf,80,stdin);
		sscanf(tmpbuf,"%u",&M134.adc134);

		if (!CheckADC(M134.adc134))
		{
		  printf ("\n%u is not a valid ADC address \n",M134.adc134);
		  printf("press ENTER to try again");

		  if (getch() == 27)
		  {
			 goto TOP_MENU;
		  }
		  goto GET_ADC;

		}

		if (ADInit(M134.adc134, M134.controlCode)==0)
		{
		  printf ("\nModel 134 not found at ADC %u\n",M134.adc134);
		  printf("press ENTER to try again, ESC to abort");

		  if (getch() == 27)
		  {
			 goto TOP_MENU;
		  }

		  goto GET_ADC;
		}

	 }

	 if (ch == '2')
	 {

		if (ADInit(M134.adc134, M134.controlCode)==0)
		{
		  clrscr();
		  printf ("\nModel 134 not found at ADC %u\n",M134.adc134);
		  printf("press any key to continue");
		  getch();
		  goto TOP_MENU;
		}

		goto POLLED;
	 }

	 if (ch == '3')
	 {

		if (ADInit(M134.adc134, M134.controlCode)==0)
		{
		  clrscr();
		  printf ("\nModel 134 not found at ADC %u\n",M134.adc134);
		  printf("press any key to continue");
		  getch();
		  goto TOP_MENU;
		}

		goto GET_SCANS;
	 }

  } //while Ch != 27

  goto EXIT_MAIN;

POLLED:

  ch = 0;
  p_row = 13;

  clrscr();
  PrintCommandMenu(&M134);
  while (ch != 27)              //repeat until esc key pressed or an error
  {

	 while (!kbhit())//loop until keyboard hit
	 {
		gotoxy(2,p_row++);
		voltage = GetConv(&M134);

		printf ("  %9.4lf",voltage);

		if (max < voltage)
		{
		  max = voltage;               //update minimum
		}

		if (min > voltage)
		{
		  min = voltage;               //update minimum
		}

		StandardDev(&stDv, &rmsBits, &sdTot, &runningAvg, sd, &voltage, &sdCount);

		gotoxy (57,4);
		if (M134.digInp == 1)
		{
		  printf("on ");
		}
		else
		{
		  printf("off");
		}

		if (p_row == 23)
		{
		  p_row = 13;

		  gotoxy(40,14);
		  printf ("Maximum     %8.4lf v       ", max);
		  gotoxy (40,15);
		  printf ("Minimum     %8.4lf v       ", min);
		  gotoxy (40,16);
		  printf ("Delta       %8.4lf v       ", fabs(max - min));
		  gotoxy (40,17);
		  printf ("Average     %8.4lf v       ", runningAvg);
		  gotoxy (40,19);
		  printf ("Standard deviation  %8.4lf v    ", stDv);
		  gotoxy (40,20);
		  printf ("Effective Resolution  %5.1lf   bits", rmsBits);
		}

	 }   //while (!kbhit())

	 ch = getch();

	 if (ch != 27)
	 {

		if (islower(ch))
		{
		  ch = toupper(ch);
		}

		DoCommand (&M134, ch, &max, &min);
	 }

  }     //while (Ch != 27)

  goto TOP_MENU;  //end of polled mode example

GET_SCANS:
	if (hglbStoreConv != NULL)
	{
	  GlobalUnlock(hglbStoreConv);
	  GlobalFree(hglbStoreConv);
	}
	if (!SetupScan(&M134, &maxHertzRate))
	{
	  goto TOP_MENU;
	}

GET_NUM_SCANS:
  clrscr();
  printf("%lu scans per sampling run (Y/N)? ",M134.totScans);
  ch = getch();

  if (ch == 27)
  {
	 return 0;
  }

  if (ch == 'N' || ch == 'n')
  {
	 printf("\n\nEnter scans per sampling run ");
	 fgets(tmpbuf,80,stdin);
	 sscanf(tmpbuf,"%lu",&M134.totScans);

	 if (M134.totScans == 0)
	 {
		M134.totScans = 1;
	 }

	 goto GET_NUM_SCANS;
  }
  else if (ch != 'Y' && ch != 'y')
  {
	 goto GET_NUM_SCANS;
  }

  scanTime = 1/M134.hertzRate * (M134.totScans-1) + 1/(maxHertzRate);

  arraySize = (M134.totScans*M134.numData)*2;  //each datapoint is 2 bytes

  hglbStoreConv=GlobalAlloc(GPTR,arraySize);
  if (hglbStoreConv == NULL)
  {
	 printf("\n\nunable to allocate %ld bytes. Reduce number of scans\n",arraySize);
	 printf("any key to continue");
	 getch();
	 goto GET_NUM_SCANS;
  }

  StoreConv = (unsigned int huge *)GlobalLock(hglbStoreConv);

DISPLAY:
	clrscr();

	printf("\nInterrupts = ");

	if (M134.interruptFlag == 1)
	{
	  printf ("on\n");
	}
	else
	{
	  printf ("off\n");
	}

	printf("Data points per scan = %u\n", M134.numData);
	printf("Average %u samples per data point\n",M134.average);
	printf("Inter-channel delay count= %u   %8.0lf microseconds\n",M134.ichDelay,(double)(M134.ichDelay*tickTime*1e6));
	printf("Scans per sample run = %lu\n", M134.totScans);
	printf("Size of array = %lu bytes\n",arraySize);
	printf("Scanning rate = %8.2lf hz\n",M134.hertzRate);
	printf("Sample run duration %8.2lf seconds\n",scanTime);
	printf("\n\nIs everything correct (Y/N)? ");
	ch = getch();

	if (ch == 27)
	{
	  goto TOP_MENU;
	}

	if (ch == 'N' || ch == 'n')
	{
	  goto GET_SCANS;
	}
	else if (ch != 'Y' && ch != 'y')
	{
	  goto DISPLAY;
	}

REVIEW_CODES:
	clrscr();
	printf ("Review or revise control codes (Y/N)? ");
	ch = getch();


	if (ch == 27)
	{
	  goto TOP_MENU;
	}

REVIEW_CODES2:

	if (ch == 'Y' || ch == 'y')
	{

	  DispCodes(&M134);

	  p_row = 4;
	  p_col = 1;
	  incFlag = 0;
	  M134.cCode[0] = -1;

	  for (n = 1;n <= M134.numData;n++)
	  {
		 gotoxy(p_col,p_row++);

		 if (incFlag == 0)
		 {

			if (M134.cCode[n] < 10)
			{
			  gotoxy(p_col+12,p_row-1);
			}
			else
			{
			  gotoxy(p_col+11,p_row-1);
			}

			fgets(tmpbuf,80,stdin);

			if (tmpbuf[0] == 'i' || tmpbuf[0] == 'I')
			{
			  incFlag = 1;
			  M134.cCode[n] = M134.cCode[n-1]+1;
			  if (M134.cCode[n] >= 64)
			  {
				 M134.cCode[n] = 0;
			  }
			  gotoxy(p_col+11,p_row-1);
			  printf ("%2u  ",M134.cCode[n]);
			}
			else
			{
			  gotoxy(p_col+12,p_row-1);
			  printf(" ");
			  gotoxy(p_col+12,p_row-1);

			  sscanf(tmpbuf,"%u",&M134.cCode[n]);

			  if (M134.cCode[n] >= 64)
			  {
				 M134.cCode[n] = 63;
			  }

			}

		 } //if incFlag == 0
		 else if (incFlag == 1)
		 {
			M134.cCode[n] = M134.cCode[n-1]+1;
			if (M134.cCode[n] >= 64)
			{
			  M134.cCode[n] = 0;
			}
		 }


		 gotoxy(p_col+11,p_row-1);
		 if ((p_col + 20) > 60)
		 {
			printf ("%2u    ",M134.cCode[n]);
		 }
		 else
		 {
			printf ("%2u       ",M134.cCode[n]);
		 }

		 if (p_row == 20 && (n != M134.numData))
		 {
			p_col += 19;
			p_row = 4;
		 }

	  }  //end for loop

	  M134.cCode[n] = M134.cCode[1];

EXIT_CODES:
	  gotoxy(10,24);
	  printf ("Revise control codes (Y/N)?                                     ");
	  gotoxy(38,24);
	  ch = getch();

	  if (ch == 27)
	  {
		 goto TOP_MENU;
	  }

	  if (ch == 'Y' || ch == 'y')
	  {
		 goto REVIEW_CODES2;
	  }
	  else if (ch != 'N' && ch != 'n')
	  {
		 goto EXIT_CODES;
	  }

	}
	else if (ch != 'N' && ch != 'n')
	{
	  goto REVIEW_CODES;
	}

	clrscr();
	printf("\nSampling will take %8.2lf seconds\n",scanTime);
	printf("press a key to start scanning, ESC to exit");

	if (getch() == 27)
	{
	  goto TOP_MENU;
	}

	gotoxy(1,3);
	printf("Activating digital input will cut scan short\n");
	gotoxy(1,4);

	xx = clock();
	cStat = Call134(&M134, StoreConv);
	xxT = (double)(clock() - xx)/CLK_TCK;
	if (cStat == 0)
	{
	  printf ("\nUnable to find Model 134\n");
	  printf("press any key to continue");
	  getch();
	  goto TOP_MENU;
	}
	else if(cStat == 2)
	{
	  printf ("Sampling stopped. ");
	}

	printf ("%lu of %lu scans completed\n",M134.numCompleted, M134.totScans);

	if(cStat == 1)
	{
	  printf ("\nSampling complete.\n");
	}
	printf ("Elapsed time:   %8.2lf",xxT);

SAVE_FILE:
	gotoxy(1,8);
	printf("Do you want to save the data as a text file (Y/N)?     ");
	gotoxy(52,8);
	ch = getch();

	if (ch == 'Y' || ch == 'y')
	{
	  SaveData(&M134, StoreConv);
	}
	else if (ch != 'N' && ch != 'n')
	{
	  goto SAVE_FILE;
	}


DISP_SCANS:
	endDisplay =  ceil((double)(M134.numData)/8.0);
	clrscr();
	gotoxy(1,25);
	printf("any key to continue, ESC to end                    ");
	gotoxy(1,1);

	numScans = M134.numCompleted;

	for (i = 0; i < (numScans); i++)
	{
	  printf("%-4ld", i+1);

	  for (j = 0; j < (M134.numData); j++)
	  {
		 voltage = (StoreConv[i*M134.numData + j]-32768.0) * 1.52588E-04;
		 printf (" %7.4lf ",voltage);

		 if (wherex() > 70)
		 {
			printf("\n    ");
		 }
	  }

	  printf ("\n");
	  if ((endDisplay + wherey()) > 24)
	  {
		 if (getch() == 27)
		 {
			goto END_SCANS;
		 }
		 gotoxy(1,1);
	  }
	}

END_SCANS:
	gotoxy(1,25);
	printf("R to retry, ESC for menu                              ");
	gotoxy(26,25);

RETRY:
	ch = getch();

	if (ch == 'R' || ch == 'r')
	{
	  goto DISPLAY;
	}
	else if( ch == 27)
	{
	  goto TOP_MENU;
	}

	goto RETRY;

EXIT_MAIN:
	FreeLibrary(hLibrary);

SAVE_CONFIG:
	clrscr();
	printf("\nDo you wish to save configuration (Y/N)? ");
	ch = getch();

	if (ch == 'Y' || ch == 'y')
	{
	  WriteConfig(&M134);
	}
	else if(ch != 'N' && ch != 'n')
	{
	  goto SAVE_CONFIG;
	}

	DestroyWindow(GetActiveWindow());
	exit(0);
	return 1;
}

void SetWindowToDesktop(void)  //make window the same size as the desktop
{
RECT rect;

  clrscr();
  GetWindowRect(GetDesktopWindow(), &rect);
  SetWindowPos(GetActiveWindow(), (HWND) NULL, 0, 0,
					rect.right, rect.bottom,
					SWP_NOZORDER | SWP_NOACTIVATE);
}

void PrintCommandMenu(T_M134 far *M134)
{

  gotoxy (15,1);
  printf ("POLLED  MENU   Lawson Labs Model 134");
  gotoxy (5,3);
  printf ("C) A/D input Channel    %u", (M134->controlCode>>4));
  gotoxy (5,4);
  printf ("E) External sub-channel %u",M134->controlCode&0xF);

  if ((M134->controlCode&0xF) < 10)
  {
	 printf (" ");
  }

  gotoxy (43,3);
  printf ("A) Averaging            %-5u", M134->average);

  gotoxy (43,4); printf ("Digital input ");
  if (M134->digInp == 1)
  {
	 printf("on ");
  }
  else
  {
	 printf("off");
  }

  gotoxy (7,11); printf ("Volts\n");
  gotoxy (1,24);
  printf (" Press ESC for menu                                              ");
}

void DoCommand(T_M134 far *M134, char ch, double *max, double *min)
{
int     tempCh, tempExt;
char    tmpbuf[80];

  if (ch == 'A')
  {
	 M134->average = GetAvg(M134->average);
  }

  if (ch == 'C')
  {
	 tempCh = -1;
	 gotoxy(1,24);
	 printf("                                              ");

	 while ((tempCh < '0') || (tempCh > '3'))
	 {
		gotoxy(1,24);
		printf("Enter channel ");
		tempCh =(short)getche();
	 }

	 M134->controlCode = ((short)(tempCh - '0')<<4) + (M134->controlCode&0xF);

	 gotoxy(1,24);
	 printf("                                              ");
  }

  if (ch == 'E')
  {
	 tempExt = -1;
	 gotoxy(1,24);
	 printf("                                              ");

	 while ((tempExt < 0) || (tempExt > 15))
	 {
		gotoxy(1,24);
		printf("Enter sub channel                           ");
		gotoxy(19,24);
		fgets(tmpbuf,80,stdin);
		sscanf(tmpbuf,"%d",&tempExt);
	 }

	 M134->controlCode = ((M134->controlCode>>4)<<4) + tempExt;

	 printf("                                              ");
  }

  if (ch == ' ')
  {
	 (*max) = -6.0;
	 (*min) = 6.0;   //reset max-min
  }

  PrintCommandMenu(M134);
}

unsigned int GetAvg(unsigned int average)
{
int tempCh;

  tempCh = 0;

  while((tempCh != 27) && (tempCh != 13))
  {

	 while (!kbhit())
	 {
		gotoxy (1,24);
		printf ( "Press + increase, - decrease averaging, Enter to continue ");
		printf ( "%-6u", average);
	 }

	 tempCh =(short) getch();
	 if (tempCh == '+')
	 {
		average *= 2;
		if (average == 0)
		{
		  average = 1;
		}
	 }

	 if (tempCh == '-')
	 {
		average /= 2;
	 }

	 if (average  < 1)
	 {
		average = 256;
	 }
	 else if (average > 256)
	 {
		average = 1;
	 }

  }
  return average;
}

void SaveData(T_M134 far *M134, unsigned int huge *dataBuffer)
{
char             tmpbuf[80], textFile[13];
long             i, j;
double           voltage;
FILE             *tFile;  //handle for saving text file to disk

  printf("\n\nEnter the name of the text file ");
  fgets(tmpbuf,80,stdin);
  sscanf(tmpbuf,"%13s",&textFile);

  if(strlen(tmpbuf))
  {
	 if ((tFile = fopen(textFile, "wt")) != NULL)
	 {
		printf("Saving data...");
		for (i = 0; i < (M134->totScans); i++)
		{
		  fprintf(tFile,"%5ld ", i+1);
		  for (j = 0; j < (M134->numData); j++)
		  {
			 voltage = (dataBuffer[i*M134->numData + j]-32768.0) * 1.52588E-04;
			 fprintf (tFile," %7.4lf ",voltage);
		  }
		  fprintf (tFile,"\n");
		}
		fclose(tFile);
		gotoxy(1,wherey());
		printf("Data saved in file %s",textFile);
	 }
	 else
	 {
		printf("unable to open file %s\nPress any key to continue",textFile);
		getch();
	 }
  }
}

void DispCodes(T_M134 far *M134)
{
int p_row, p_col, n;

  clrscr();
  gotoxy(1,1);
  printf("  data\n");
  printf("  point   CCode\n");
  gotoxy(10,24);
  printf("Press RETURN to leave unchanged");

  if (M134->numData > 1)
  {
	 printf(", I to increment ");
  }

  p_row = 4;
  p_col = 1;
  for (n = 1;n <= M134->numData;n++)
  {
	 gotoxy(p_col,p_row++);
	 printf("   %2d      %2u  ", n, M134->cCode[n]);
	 if (p_row == 20 && (n != M134->numData))
	 {
		p_col += 19;
		p_row = 4;
		gotoxy(p_col, 1);
		printf("  data");
		gotoxy(p_col, 2);
		printf("  point   CCode");
	 }

  }
  M134->cCode[n] = M134->cCode[1];
}

short ReadConfig(T_M134 far *M134)
{
int n;
char tmpbuf[80];
FILE *rCfg;

	if ((rCfg = fopen("134ewin.cfg", "rt")) != NULL)
	{
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%u",&M134->adc134);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%lf",&M134->hertzRate);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%u",&M134->average);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%u",&M134->ichDelay);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%u",&M134->numData);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%lu",&M134->totScans);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%u",&M134->controlCode);
	  for (n = 0;n<= 65;n++)
	  {
		 fgets(tmpbuf,80,rCfg);
		 sscanf(tmpbuf,"%u",&M134->cCode[n]);
	  }
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%u",&M134->interruptFlag);

	  fclose(rCfg);
	  return 1;
	}
	else
	{
	  M134->adc134 = 797;
	  M134->hertzRate = 10000;
	  M134->ichDelay = 0;
	  M134->average = 1;
	  M134->numData = 1;
	  M134->totScans = 100;
	  M134->controlCode = 0;
	  for (n=0;n<=65;n++)
	  {
		 M134->cCode[n] = 0;
	  }
	  M134->interruptFlag = 1;

	  return 0;
	}

}


short WriteConfig(T_M134 far *M134)
{
int n;
FILE *wCfg;

  if ((wCfg = fopen("134ewin.cfg", "wt")) != NULL)
  {
	 fprintf(wCfg,"%u\n",M134->adc134);
	 fprintf(wCfg,"%lf\n",M134->hertzRate);
	 fprintf(wCfg,"%u\n",M134->average);
	 fprintf(wCfg,"%u\n",M134->ichDelay);
	 fprintf(wCfg,"%u\n",M134->numData);
	 fprintf(wCfg,"%lu\n",M134->totScans);
	 fprintf(wCfg,"%u\n",M134->controlCode);
	 for (n = 0;n<= 65;n++)
	 {
		fprintf(wCfg,"%u\n",M134->cCode[n]);
	 }
	 fprintf(wCfg,"%u\n",M134->interruptFlag);
	 fclose(wCfg);
	 return 1;
  }

  return 0;
}

short            SetupScan(T_M134 far *M134,
							  double *maxHertzRate)
{
char ch, tmpbuf[80];

GET_INTERRUPT:
  clrscr();
  printf("Interrupts = ");
  if (M134->interruptFlag == 1)
  {
	 printf ("on");
  }
  else
  {
	 printf ("off");
  }
  printf (" (Y/N)? ");

  ch = getch();

  if (ch == 27)
  {
	  return 0;
  }

  if (ch == 'N' || ch == 'n')
  {

	 if (M134->interruptFlag == 1)
	 {
		M134->interruptFlag = 0;
	 }
	 else
	 {
		M134->interruptFlag = 1;
	 }

	 goto GET_INTERRUPT;
  }
  else if (ch != 'Y' && ch != 'y')
  {
	 goto GET_INTERRUPT;
  }

GET_POINTS:
  clrscr();
  printf("data points/scan= %u (Y/N)? ",M134->numData);
  ch = getch();

  if (ch == 27)
  {
	 return 0;
  }

  if (ch == 'N' || ch == 'n')
  {
	 printf("\n\nEnter number of data points per scan ");
	 fgets(tmpbuf,80,stdin);
	 sscanf(tmpbuf,"%u",&M134->numData);

	 if (M134->numData == 0)
	 {
		M134->numData = 1;
	 }
	 else if (M134->numData > 64)
	 {
		M134->numData = 64;
	 }

	 goto GET_POINTS;
  }
  else if (ch != 'Y' && ch != 'y')
  {
	 goto GET_POINTS;
  }

GET_AVG:
  clrscr();
  printf("average %u samples/data point (Y/N)? ",M134->average);
  ch = getch();

  if (ch == 27)
  {
	 return 0;
  }

  if (ch == 'N' || ch == 'n')
  {
	 M134->average = GetAvg(M134->average);
	 goto GET_AVG;
  }
  else if (ch != 'Y' && ch != 'y')
  {
	 goto GET_AVG;
  }

  if (M134->numData == 1)
  {
	 M134->ichDelay = 0;
  }
  else if (M134->numData > 1)
  {
GET_ICH:
	 clrscr();
	 printf("interchannel delay count=  %u  ticks  %8.0lf microseconds (Y/N)? ",M134->ichDelay,(double)(M134->ichDelay*tickTime*1e6));
	 ch = getch();

	 if (ch == 27)
	 {
		return 0;
	 }

	 if (ch == 'N' || ch == 'n')
	 {
		printf("\n\nEnter Interchannel delay count ");
		fgets(tmpbuf,80,stdin);
		sscanf(tmpbuf,"%u",&M134->ichDelay);
		goto GET_ICH;
	 }
	 else if (ch != 'Y' && ch != 'y')
	 {
		goto GET_ICH;
	 }
  }

GET_RATE:
	clrscr();

	if (OverHead(M134) == 0)
	{
	 printf ("\nUnable to find Model 134\n");
	 printf("press any key to continue");
	 getch();
	 return 0;
  }

  *maxHertzRate = 1/((long)((long)(M134->ichDelay+M134->average*(convTime+M134->overHead))*(long)M134->numData)*tickTime);

  if ((M134->hertzRate+.005) > *maxHertzRate)
  {
	 printf("%8.2lf Max Rate (Y/N)? ",*maxHertzRate);
  }
  else
  {
	 printf("%8.2lf scan rate (Y/N)? ",M134->hertzRate);
  }

  ch = getch();

  if (ch == 27)
  {
	 return 0;
  }

  if (ch == 'N' || ch == 'n')
  {
	 printf ("\n\nMax Rate %8.2lf",*maxHertzRate);
	 printf("\n\nEnter scan rate ");
	 fgets(tmpbuf,80,stdin);
	 sscanf(tmpbuf,"%lf",&M134->hertzRate);

	 if (M134->hertzRate <= 0)
	 {
		M134->hertzRate = 1;
	 }

	 M134->hertzRate = 1/(((long)((1/M134->hertzRate)/ tickTime))*tickTime);

	 goto GET_RATE;
  }
  else if (ch != 'Y' && ch != 'y')
  {
	 goto GET_RATE;
  }

  if (M134->hertzRate > *maxHertzRate)
  {
	 M134->hertzRate = *maxHertzRate;
  }

  return 1;
}

short CheckADC(unsigned int adc134)
{
  if (adc134 == 769 ||
		adc134 == 773 ||
		adc134 == 777 ||
		adc134 == 781 ||
		adc134 == 785 ||
		adc134 == 789 ||
		adc134 == 793 ||
		adc134 == 797)
  {
	 return 1;
  }
  return 0;
}

void    StandardDev(double *stDv,
						  double *rmsBits,
						  double *sdTot,
						  double *runningAvg,
						  double *sd,
						  double *voltage,
						  int    *sdCount)
{//standard deviation and RMS resolution calculation
int i = 0, nnn, sdGrp=40;;
double tempStDv, StDvRms;

  sd[*sdCount] = (*voltage);
  (*sdTot) += (*voltage);

  if ((*sdCount) == sdGrp)
  {
	 (*stDv) = 0; (*sdTot) = (*sdTot) / sdGrp;

	 for (nnn = 1; nnn != sdGrp; nnn++)
	 {
		(*stDv) = (*stDv) + pow(((*sdTot) - sd[nnn]), 2);
	 }

	 (*stDv) = sqrt((*stDv) / sdGrp);
	 StDvRms = (*stDv) / 1.52588E-04;  //.001192093;
	 tempStDv = StDvRms;

CALCNEXT:

	 if (tempStDv >= 1)
	 {
		i = i + 1;
		tempStDv = tempStDv / 2;
		goto CALCNEXT;
	 }

	 (*rmsBits) = 16 - i;

	 if ((tempStDv < .75) && (i != 0))
	 {
		(*rmsBits) = (*rmsBits) + .5;
	 }

	 (*runningAvg) = (*sdTot);
	 (*sdTot) = 0; (*sdCount) = 1;
  }
  else
  {
	 (*sdCount)++;
  }

}

